/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Hashinator Extension.
 *
 * The Initial Developer of the Original Code is
 * Patrick L. Woodworth.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

String.prototype.trim = function() {
    return( this.replace( /^\s+/, '' ).replace( /\s+$/, '' ) );
}

String.prototype.rotateRight = function() {
    return "" + this.charAt( this.length - 1 ) + this.substring( 0, this.length - 1 );
}



function Hashinator() {
    this._init_();
}

////////////////////////////////////////////////////////////////////////
// BEGIN: Constants
////////////////////////////////////////////////////////////////////////

var PWD_HOST = "about:blank";

var PWD_USER = "Hashinator";

////////////////////////////////////////////////////////////////////////
// END: Constants
////////////////////////////////////////////////////////////////////////

Hashinator.prototype= {

    debug_counter: 0,

    ////////////////////////////////////////////////////////////////////////
    // BEGIN: Constructors
    ////////////////////////////////////////////////////////////////////////

    _init_: function() {
        var prefs = this.m_prefs;
        var hashversion = prefs.getIntPref("mode");
        if (hashversion == 0) {
            prefs.setIntPref("mode",1);
        }
    },

    ////////////////////////////////////////////////////////////////////////
    // END: Constructors
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    // BEGIN: Accessors
    ////////////////////////////////////////////////////////////////////////

    get m_prefs() {
        return Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefService).getBranch("hashinator.");
    },

    get m_maxlength() {
        return this.m_prefs.getIntPref("maxlength");
    },

    get m_autofill() {
        return this.m_prefs.getBoolPref("autofill");
    },

    set m_autofill(val) {
        this.m_prefs.setBoolPref("autofill",val);
    },

    get m_password() {
        var host= new Object();
        var user= new Object();
        var pass= new Object();
        try{
            var pmgr = Components.classes["@mozilla.org/passwordmanager;1"]
                    .createInstance(Components.interfaces.nsIPasswordManagerInternal);
            pmgr.findPasswordEntry(PWD_HOST,PWD_USER,"",host,user,pass);
            return pass.value;
        }catch(e){
        }
        return "";
    },

    set m_password(val) {
        var pmgr = Components.classes["@mozilla.org/passwordmanager;1"]
                .createInstance(Components.interfaces.nsIPasswordManager);
        try{
            pmgr.removeUser(PWD_HOST,PWD_USER);
        }catch(e){
        }
        pmgr.addUser(PWD_HOST,PWD_USER, val );
    },

    isValidPassword: function() {
        try {
            var password = this.m_password;
            if ((password == null) || (password.length == 0))
                return false;
            return true;
        } catch (e) {
//            alert( e );
        }
        return false;
    },

    ////////////////////////////////////////////////////////////////////////
    // END: Accessors
    ////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////
    // BEGIN: Statics -- It's not safe to use the 'this' keyword with these.
    ////////////////////////////////////////////////////////////////////////

    s_onLoadEvent: function(event) {
        document.getElementById("contentAreaContextMenu")
                .addEventListener("popupshowing", gHashinator.s_onPopupShowing, false);
    },

    s_onLoadEvent2: function(event) {
        try {
            if (gHashinator.m_autofill && gHashinator.isValidPassword()) {
                gHashinator.genFromPageToPage();
            }
        } catch (e) {
            alert( e );
        }
    },

    s_onPopupShowing: function(event) {
        var cm = gContextMenu;
        document.getElementById("context-hashpass-p2p").hidden = !(cm.onTextInput);
    },

    ////////////////////////////////////////////////////////////////////////
    // END: Statics
    ////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////
    // BEGIN: Public methods
    ////////////////////////////////////////////////////////////////////////

    genFromPageToPage: function() {
        try {
            var domain = this.extractDomainFromPage()
            var passhash = this.hashinateForDomain( domain );
            this.fillFormPasswords( passhash );
        } catch (e) {
            alert( e );
        }
    },

    genFromPageToClip: function() {
        try {
            var domain = this.extractDomainFromPage();
            var passhash = this.hashinateForDomain( domain );
            this.copyToClip( passhash );
        } catch (e) {
            alert( e );
        }
    },

    genFromOptionsToClip: function() {
        try {
            var domain = document.getElementById("hashinator-opt-domain").value;
            var passhash = this.hashinateForDomain( domain );
            this.copyToClip( passhash );
        } catch (e) {
            alert( e );
        }
    },

    initOptions: function() {
        var passwd = this.m_password;
        document.getElementById("hashinator-opt-password").value = passwd;
        document.getElementById("hashinator-opt-pwconfirm").value = passwd;
        document.getElementById("hashinator-opt-autofill").checked = this.m_autofill;
    },

    applyOptions: function() {
        try {
            var passwd = document.getElementById("hashinator-opt-password").value;
            if (passwd != document.getElementById("hashinator-opt-pwconfirm").value) {
                alert( "Passphrases do not match!" );
                return false;
            }
            this.m_password = passwd;
            this.m_autofill = document.getElementById("hashinator-opt-autofill").checked;
            return true;
        } catch (e) {
            alert( e );
        }
        return false;
    },

    saveOptions: function() {
        return this.applyOptions();
    },

    ////////////////////////////////////////////////////////////////////////
    // END: Public methods
    ////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////
    // BEGIN: Private methods
    ////////////////////////////////////////////////////////////////////////

    extractDomainFromPage: function() {
        try {
            var h = window._content.document.location.host.split('.');
            var p = h.length;
            return '' + h[p-2] + '.' + h[p-1];
        } catch (e) {
        }
        return "";
    },


    copyToClip: function( meintext ) {
        try {
            // maak een interface naar het clipboard
            var clip = Components.classes['@mozilla.org/widget/clipboard;1']
                    .createInstance(Components.interfaces.nsIClipboard);

            // maak een transferable
            var trans = Components.classes['@mozilla.org/widget/transferable;1']
                    .createInstance(Components.interfaces.nsITransferable);

            // specificeer wat voor soort data we op willen halen; text in dit geval
            trans.addDataFlavor('text/unicode');

            // om de data uit de transferable te halen hebben we 2 nieuwe objecten nodig   om het in op te slaan
            var str = new Object();
            var len = new Object();

            var str = Components.classes["@mozilla.org/supports-string;1"]
                    .createInstance(Components.interfaces.nsISupportsString);

            var copytext = meintext;

            str.data = copytext;

            trans.setTransferData("text/unicode",str,copytext.length*2);

            var clipid=Components.interfaces.nsIClipboard;

            clip.setData(trans,null,clipid.kGlobalClipboard);

        } catch (e) {
            alert( e );
            return false;
        }
//            alert( "Following info was copied to your clipboard:\n\n" + meintext);
        return true;
    },

    fillFormPasswords: function( passwd ) {
        if (passwd == null)
            return;
        try {
            var i=0, j=0, F=window._content.document.forms;
            for(i=0; i < F.length; i++) {
                var E = F[i].elements;
                for(j=0; j < E.length; j++) {
                    var D = E[j];
                    if(D.type=='password') {
                        if (D.value.length < 1)
                            D.value=passwd;
                    }
                }
            }
        } catch (e) {
            alert( e );
        }
    },

    hashinateForDomain: function( domain ) {
        var password = this.m_password;
        if (password.length == 0) {
            throw "You must set a hashinator password before use!";
        }
        var retval = this.hashinate( password + ":" + domain );
        return retval;
    },

    hashinate: function( rawpass ) {

        var maxlen = this.m_maxlength;
        var rawhash = b64_md5( rawpass );

        // make sure we have a hash with both alpha and numeric chars
        var goodhash = false;
        while (!goodhash) {
            rawhash = b64_md5( rawhash );
            goodhash = true;
            if (rawhash.search(/\d/) < 0)
                goodhash = false;
            if (rawhash.search(/[A-Za-z]/) < 0)
                goodhash = false;
        }

        // get rid of non-alphanumeric characters
        var rothash = rawhash.replace(/[\+\/\=]/g,'');

        var numIdx = rothash.search(/\d/);
        while ((numIdx == 0) || (numIdx >= maxlen)) {
            rothash = rothash.rotateRight();
            numIdx = rothash.search(/\d/);
        }

        var retval = rothash;
        return retval.substring(0,maxlen);
    },

    foobarbang: function() {
    }

    ////////////////////////////////////////////////////////////////////////
    // END: Private methods
    ////////////////////////////////////////////////////////////////////////
}

var gHashinator = new Hashinator();


window.addEventListener("load", gHashinator.s_onLoadEvent, false);
window.addEventListener("load", gHashinator.s_onLoadEvent2, true);
